/*
ID: espr1t
TASK: Decoration
KEYWORDS: NP, Ad-hoc
CONTEST: CodeIT 2013-2014, Round 3
*/

#include <cstdio>
#include <ctime>
#include <cstring>
#include <cassert>
#include <vector>
#include <algorithm>
#include <queue>
#include <set>

using namespace std;
 
const int INF = 1000000001;

const int MAX_SIZE = 10002;
const double MAX_EXEC_TIME = 4.2; // Seconds



unsigned startTime;
double execTime = 0.0;
inline double getTime() {
    return execTime = (double)(clock() - startTime) / CLOCKS_PER_SEC;
}


struct Window {
    int height, width;
    vector < vector <int> > board;
};

struct Form {
    int height, width;
    int filled;
    double likes;
    int win, row, col;
    vector < vector <int> > board;
};

vector <Form> forms;
vector <Window> windows;


/*
 * Tries to fit a form into a window. Returns a heuristic how much space is
 * "wasted" if the form is placed on this window.
 */
int fitFormInWindow(int formIdx, int windowIdx, bool update = false) {
    Form& form = forms[formIdx];
    Window& window = windows[windowIdx];
    
    int best = INF;
    int atRow = -1, atCol = -1;
    for (int srow = 0; srow + form.height <= window.height; srow++) {
        for (int scol = 0; scol + form.width <= window.width; scol++) {
            bool okay = true;
            for (int row = 0; row < form.height; row++) {
                for (int col = 0; col < form.width; col++) {
                    if (form.board[row][col] && window.board[srow + row][scol + col]) {
                        okay = false;
                        break;
                    }
                }
                if (!okay) break;
            }
            if (!okay) continue;

            int penalty = 1;
            // By row
            for (int row = 0; row < form.height; row++) {
                int first = -1, last = -1;
                for (int col = 0; col < form.width; col++) {
                    if (form.board[row][col]) {
                        if (first == -1)
                            first = col;
                        last = col;
                    }
                }
                int left = 0, right = 0;
                for (int i = scol + first - 1; i >= 0; i--) {
                    if (window.board[srow + row][i]) break;
                    left++;
                }
                for (int i = scol + last + 1; i < window.width; i++) {
                    if (window.board[srow + row][i]) break;
                    right++;
                }
                penalty += min(left, right);
            }
            // By column
            for (int col = 0; col < form.width; col++) {
                int first = -1, last = -1;
                for (int row = 0; row < form.height; row++) {
                    if (form.board[row][col]) {
                        if (first == -1)
                            first = row;
                        last = row;
                    }
                }
                int up = 0, down = 0;
                for (int i = srow + first - 1; i >= 0; i--) {
                    if (window.board[i][scol + col]) break;
                    up++;
                }
                for (int i = srow + last + 1; i < window.height; i++) {
                    if (window.board[i][scol + col]) break;
                    down++;
                }
                penalty += min(up, down);
            }
            if (best > penalty)
                best = penalty, atRow = srow, atCol = scol;
        }
    }
    if (best < INF && update) {
        form.win = windowIdx, form.row = atRow, form.col = atCol;
        for (int row = 0; row < form.height; row++)
            for (int col = 0; col < form.width; col++)
                if (form.board[row][col])
                    window.board[atRow + row][atCol + col] = formIdx + 1;
    }
    return best;
}


void solve() {
    vector <bool> used(forms.size(), false);
    vector < vector <int> > score(forms.size(),
        vector <int>(windows.size(), INF));
    
    for (int i = 0; i < (int)forms.size(); i++)
        for (int c = 0; c < (int)windows.size(); c++)
            score[i][c] = fitFormInWindow(i, c);
    
    for (int iter = 0; iter < (int)forms.size(); iter++) {
        if (getTime() > MAX_EXEC_TIME)
            break;
        
        // Find best form to use.
        double value = -INF; int form = -1;
        for (int i = 0; i < (int)forms.size(); i++) if (!used[i]) {
            double cvalue = forms[i].likes / forms[i].height / forms[i].width;
//            double cvalue = forms[i].likes / forms[i].filled;
            if (value < cvalue)
                value = cvalue, form = i;
        }
        
        // Cannot fit any other form.
        if (form == -1)
            break;
        used[form] = true;
        
        int window = -1;
        for (int i = 0; i < (int)windows.size(); i++) {
            if (score[form][i] < INF)
                if (window == -1 || windows[window].height * windows[window].width <
                    windows[i].height * windows[i].width) window = i;
        }

        // Cannot fit this form into any window.
        if (window == -1)
            continue;
        
        // Update window with new form.
        fitFormInWindow(form, window, true);

        // Update other forms with this window.
        for (int i = 0; i < (int)forms.size(); i++)
            if (!used[i]) score[i][window] = fitFormInWindow(i, window);
    }
}


void readInput(const char* fileName) {
    FILE* in = fopen(fileName, "rt");
    
    int numWindows, numForms;
    fscanf(in, "%d %d", &numWindows, &numForms);
    
    windows.resize(numWindows);
    for (int i = 0; i < numWindows; i++) {
        fscanf(in, "%d %d", &windows[i].height, &windows[i].width);
        windows[i].board.resize(windows[i].height, vector <int>(windows[i].width, 0));
    }
    
    forms.resize(numForms);
    for (int i = 0; i < numForms; i++) {
        forms[i].win = forms[i].row = forms[i].col = -2;
        fscanf(in, "%d %d %lf", &forms[i].height, &forms[i].width, &forms[i].likes);
        forms[i].board.resize(forms[i].height, vector <int>(forms[i].width, 0));
        forms[i].filled = 0;
        for (int row = 0; row < forms[i].height; row++) {
            for (int col = 0; col < forms[i].width; col++) {
                fscanf(in, "%d", &forms[i].board[row][col]);
                if (forms[i].board[row][col] == 1)
                    forms[i].filled++;
            }
        }
    }
    fclose(in);
}


void printOutput(const char* fileName) {
//    for (int i = 0; i < (int)windows.size(); i++)
//        printWindow(i);
    
    FILE* out = fopen(fileName, "wt");
    for (int i = 0; i < (int)forms.size(); i++)
        fprintf(out, "%d %d %d\n", forms[i].win + 1, forms[i].row + 1, forms[i].col + 1);
    fclose(out);

    double total = 0.0;
    for (int i = 0; i < (int)windows.size(); i++) {
        double sum = 0.0, cnt = 0.0;
        for (int c = 0; c < (int)forms.size(); c++)
            if (forms[c].win == i) sum += forms[c].likes, cnt += 1.0;
        total += sum * cnt;
    }
//    fprintf(stderr, "Found answer has value %.0lf\n", total);
    // 6329
    // 7853
    // 8282
    // 8596
}


void init() {
    srand(42);
    startTime = clock();

    windows.clear();
    forms.clear();
}


int main(void) {
    init();
    readInput("decoration.in");
    solve();
    printOutput("decoration.out");
    return 0;
}
